from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

if TYPE_CHECKING:
    from ..models.feedback_rating_info import FeedbackRatingInfo


T = TypeVar("T", bound="TraceRecordWithChildrenFeedbackRatingInfo")


@_attrs_define
class TraceRecordWithChildrenFeedbackRatingInfo:
    """Feedback information related to the trace"""

    additional_properties: dict[str, "FeedbackRatingInfo"] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        field_dict: dict[str, Any] = {}
        for prop_name, prop in self.additional_properties.items():
            field_dict[prop_name] = prop.to_dict()

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.feedback_rating_info import FeedbackRatingInfo

        d = dict(src_dict)
        trace_record_with_children_feedback_rating_info = cls()

        additional_properties = {}
        for prop_name, prop_dict in d.items():
            additional_property = FeedbackRatingInfo.from_dict(prop_dict)

            additional_properties[prop_name] = additional_property

        trace_record_with_children_feedback_rating_info.additional_properties = additional_properties
        return trace_record_with_children_feedback_rating_info

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> "FeedbackRatingInfo":
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: "FeedbackRatingInfo") -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, Optional, Union

import httpx

from galileo_core.constants.request_method import RequestMethod
from galileo_core.helpers.api_client import ApiClient

from ... import errors
from ...models.http_validation_error import HTTPValidationError
from ...models.list_prompt_template_version_params import ListPromptTemplateVersionParams
from ...models.list_prompt_template_version_response import ListPromptTemplateVersionResponse
from ...types import UNSET, Response, Unset


def _get_kwargs(
    template_id: str,
    *,
    body: ListPromptTemplateVersionParams,
    starting_token: Union[Unset, int] = 0,
    limit: Union[Unset, int] = 100,
) -> dict[str, Any]:
    headers: dict[str, Any] = {}

    params: dict[str, Any] = {}

    params["starting_token"] = starting_token

    params["limit"] = limit

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: dict[str, Any] = {
        "method": RequestMethod.POST,
        "return_raw_response": True,
        "path": f"/templates/{template_id}/versions/query",
        "params": params,
    }

    _kwargs["json"] = body.to_dict()

    headers["Content-Type"] = "application/json"

    _kwargs["content_headers"] = headers
    return _kwargs


def _parse_response(
    *, client: ApiClient, response: httpx.Response
) -> Optional[Union[HTTPValidationError, ListPromptTemplateVersionResponse]]:
    if response.status_code == 200:
        response_200 = ListPromptTemplateVersionResponse.from_dict(response.json())

        return response_200
    if response.status_code == 422:
        response_422 = HTTPValidationError.from_dict(response.json())

        return response_422
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: ApiClient, response: httpx.Response
) -> Response[Union[HTTPValidationError, ListPromptTemplateVersionResponse]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    template_id: str,
    *,
    client: ApiClient,
    body: ListPromptTemplateVersionParams,
    starting_token: Union[Unset, int] = 0,
    limit: Union[Unset, int] = 100,
) -> Response[Union[HTTPValidationError, ListPromptTemplateVersionResponse]]:
    """Query Template Versions

     Query versions of a specific prompt template.

    Parameters
    ----------
    template_id : UUID4
        ID of the template to query versions for
    params : ListPromptTemplateVersionParams
        Query parameters for filtering and sorting
    pagination : PaginationRequestMixin
        Pagination parameters
    ctx : Context
        User context containing database session and user information

    Returns
    -------
    ListPromptTemplateVersionResponse
        Paginated list of template version responses

    Args:
        template_id (str):
        starting_token (Union[Unset, int]):  Default: 0.
        limit (Union[Unset, int]):  Default: 100.
        body (ListPromptTemplateVersionParams):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[HTTPValidationError, ListPromptTemplateVersionResponse]]
    """

    kwargs = _get_kwargs(template_id=template_id, body=body, starting_token=starting_token, limit=limit)

    response = client.request(**kwargs)

    return _build_response(client=client, response=response)


def sync(
    template_id: str,
    *,
    client: ApiClient,
    body: ListPromptTemplateVersionParams,
    starting_token: Union[Unset, int] = 0,
    limit: Union[Unset, int] = 100,
) -> Optional[Union[HTTPValidationError, ListPromptTemplateVersionResponse]]:
    """Query Template Versions

     Query versions of a specific prompt template.

    Parameters
    ----------
    template_id : UUID4
        ID of the template to query versions for
    params : ListPromptTemplateVersionParams
        Query parameters for filtering and sorting
    pagination : PaginationRequestMixin
        Pagination parameters
    ctx : Context
        User context containing database session and user information

    Returns
    -------
    ListPromptTemplateVersionResponse
        Paginated list of template version responses

    Args:
        template_id (str):
        starting_token (Union[Unset, int]):  Default: 0.
        limit (Union[Unset, int]):  Default: 100.
        body (ListPromptTemplateVersionParams):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[HTTPValidationError, ListPromptTemplateVersionResponse]
    """

    return sync_detailed(
        template_id=template_id, client=client, body=body, starting_token=starting_token, limit=limit
    ).parsed


async def asyncio_detailed(
    template_id: str,
    *,
    client: ApiClient,
    body: ListPromptTemplateVersionParams,
    starting_token: Union[Unset, int] = 0,
    limit: Union[Unset, int] = 100,
) -> Response[Union[HTTPValidationError, ListPromptTemplateVersionResponse]]:
    """Query Template Versions

     Query versions of a specific prompt template.

    Parameters
    ----------
    template_id : UUID4
        ID of the template to query versions for
    params : ListPromptTemplateVersionParams
        Query parameters for filtering and sorting
    pagination : PaginationRequestMixin
        Pagination parameters
    ctx : Context
        User context containing database session and user information

    Returns
    -------
    ListPromptTemplateVersionResponse
        Paginated list of template version responses

    Args:
        template_id (str):
        starting_token (Union[Unset, int]):  Default: 0.
        limit (Union[Unset, int]):  Default: 100.
        body (ListPromptTemplateVersionParams):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[HTTPValidationError, ListPromptTemplateVersionResponse]]
    """

    kwargs = _get_kwargs(template_id=template_id, body=body, starting_token=starting_token, limit=limit)

    response = await client.arequest(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    template_id: str,
    *,
    client: ApiClient,
    body: ListPromptTemplateVersionParams,
    starting_token: Union[Unset, int] = 0,
    limit: Union[Unset, int] = 100,
) -> Optional[Union[HTTPValidationError, ListPromptTemplateVersionResponse]]:
    """Query Template Versions

     Query versions of a specific prompt template.

    Parameters
    ----------
    template_id : UUID4
        ID of the template to query versions for
    params : ListPromptTemplateVersionParams
        Query parameters for filtering and sorting
    pagination : PaginationRequestMixin
        Pagination parameters
    ctx : Context
        User context containing database session and user information

    Returns
    -------
    ListPromptTemplateVersionResponse
        Paginated list of template version responses

    Args:
        template_id (str):
        starting_token (Union[Unset, int]):  Default: 0.
        limit (Union[Unset, int]):  Default: 100.
        body (ListPromptTemplateVersionParams):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[HTTPValidationError, ListPromptTemplateVersionResponse]
    """

    return (
        await asyncio_detailed(
            template_id=template_id, client=client, body=body, starting_token=starting_token, limit=limit
        )
    ).parsed



# mypy: disable-error-code=syntax
# We need to ignore syntax errors until https://github.com/python/mypy/issues/17535 is resolved.
from typing import Any, Optional

from galileo_core.schemas.base_config import GalileoConfig


class GalileoPythonConfig(GalileoConfig):
    # Config file for this project.
    config_filename: str = "galileo-python-config.json"

    def reset(self) -> None:
        global _galileo_config
        _galileo_config = None

        super().reset()

    @classmethod
    def get(cls, **kwargs: Any) -> "GalileoPythonConfig":
        global _galileo_config
        _galileo_config = cls._get(_galileo_config, **kwargs)  # type: ignore[arg-type]
        return _galileo_config


_galileo_config: Optional[GalileoPythonConfig] = None



from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.scorer_created_at_filter import ScorerCreatedAtFilter
    from ..models.scorer_creator_filter import ScorerCreatorFilter
    from ..models.scorer_model_type_filter import ScorerModelTypeFilter
    from ..models.scorer_name_filter import ScorerNameFilter
    from ..models.scorer_tags_filter import ScorerTagsFilter
    from ..models.scorer_type_filter import ScorerTypeFilter
    from ..models.scorer_updated_at_filter import ScorerUpdatedAtFilter


T = TypeVar("T", bound="ListScorersRequest")


@_attrs_define
class ListScorersRequest:
    """
    Attributes:
        filters (Union[Unset, list[Union['ScorerCreatedAtFilter', 'ScorerCreatorFilter', 'ScorerModelTypeFilter',
            'ScorerNameFilter', 'ScorerTagsFilter', 'ScorerTypeFilter', 'ScorerUpdatedAtFilter']]]):
    """

    filters: Union[
        Unset,
        list[
            Union[
                "ScorerCreatedAtFilter",
                "ScorerCreatorFilter",
                "ScorerModelTypeFilter",
                "ScorerNameFilter",
                "ScorerTagsFilter",
                "ScorerTypeFilter",
                "ScorerUpdatedAtFilter",
            ]
        ],
    ] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.scorer_created_at_filter import ScorerCreatedAtFilter
        from ..models.scorer_creator_filter import ScorerCreatorFilter
        from ..models.scorer_model_type_filter import ScorerModelTypeFilter
        from ..models.scorer_name_filter import ScorerNameFilter
        from ..models.scorer_tags_filter import ScorerTagsFilter
        from ..models.scorer_type_filter import ScorerTypeFilter

        filters: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.filters, Unset):
            filters = []
            for filters_item_data in self.filters:
                filters_item: dict[str, Any]
                if isinstance(filters_item_data, ScorerNameFilter):
                    filters_item = filters_item_data.to_dict()
                elif isinstance(filters_item_data, ScorerTypeFilter):
                    filters_item = filters_item_data.to_dict()
                elif isinstance(filters_item_data, ScorerModelTypeFilter):
                    filters_item = filters_item_data.to_dict()
                elif isinstance(filters_item_data, ScorerTagsFilter):
                    filters_item = filters_item_data.to_dict()
                elif isinstance(filters_item_data, ScorerCreatorFilter):
                    filters_item = filters_item_data.to_dict()
                elif isinstance(filters_item_data, ScorerCreatedAtFilter):
                    filters_item = filters_item_data.to_dict()
                else:
                    filters_item = filters_item_data.to_dict()

                filters.append(filters_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if filters is not UNSET:
            field_dict["filters"] = filters

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.scorer_created_at_filter import ScorerCreatedAtFilter
        from ..models.scorer_creator_filter import ScorerCreatorFilter
        from ..models.scorer_model_type_filter import ScorerModelTypeFilter
        from ..models.scorer_name_filter import ScorerNameFilter
        from ..models.scorer_tags_filter import ScorerTagsFilter
        from ..models.scorer_type_filter import ScorerTypeFilter
        from ..models.scorer_updated_at_filter import ScorerUpdatedAtFilter

        d = dict(src_dict)
        filters = []
        _filters = d.pop("filters", UNSET)
        for filters_item_data in _filters or []:

            def _parse_filters_item(
                data: object,
            ) -> Union[
                "ScorerCreatedAtFilter",
                "ScorerCreatorFilter",
                "ScorerModelTypeFilter",
                "ScorerNameFilter",
                "ScorerTagsFilter",
                "ScorerTypeFilter",
                "ScorerUpdatedAtFilter",
            ]:
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    filters_item_type_0 = ScorerNameFilter.from_dict(data)

                    return filters_item_type_0
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    filters_item_type_1 = ScorerTypeFilter.from_dict(data)

                    return filters_item_type_1
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    filters_item_type_2 = ScorerModelTypeFilter.from_dict(data)

                    return filters_item_type_2
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    filters_item_type_3 = ScorerTagsFilter.from_dict(data)

                    return filters_item_type_3
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    filters_item_type_4 = ScorerCreatorFilter.from_dict(data)

                    return filters_item_type_4
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    filters_item_type_5 = ScorerCreatedAtFilter.from_dict(data)

                    return filters_item_type_5
                except:  # noqa: E722
                    pass
                if not isinstance(data, dict):
                    raise TypeError()
                filters_item_type_6 = ScorerUpdatedAtFilter.from_dict(data)

                return filters_item_type_6

            filters_item = _parse_filters_item(filters_item_data)

            filters.append(filters_item)

        list_scorers_request = cls(filters=filters)

        list_scorers_request.additional_properties = d
        return list_scorers_request

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

if TYPE_CHECKING:
    from ..models.aggregated_trace_view_graph import AggregatedTraceViewGraph


T = TypeVar("T", bound="AggregatedTraceViewResponse")


@_attrs_define
class AggregatedTraceViewResponse:
    """
    Attributes:
        graph (AggregatedTraceViewGraph):
        num_sessions (int): Number of sessions in the aggregated view
        num_traces (int): Number of traces in the aggregated view
    """

    graph: "AggregatedTraceViewGraph"
    num_sessions: int
    num_traces: int
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        graph = self.graph.to_dict()

        num_sessions = self.num_sessions

        num_traces = self.num_traces

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({"graph": graph, "num_sessions": num_sessions, "num_traces": num_traces})

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.aggregated_trace_view_graph import AggregatedTraceViewGraph

        d = dict(src_dict)
        graph = AggregatedTraceViewGraph.from_dict(d.pop("graph"))

        num_sessions = d.pop("num_sessions")

        num_traces = d.pop("num_traces")

        aggregated_trace_view_response = cls(graph=graph, num_sessions=num_sessions, num_traces=num_traces)

        aggregated_trace_view_response.additional_properties = d
        return aggregated_trace_view_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="SystemMetricInfo")


@_attrs_define
class SystemMetricInfo:
    """
    Attributes:
        name (str):
        max_ (Union[None, Unset, float]):
        mean (Union[None, Unset, float]):
        median (Union[None, Unset, float]):
        min_ (Union[None, Unset, float]):
        p25 (Union[None, Unset, float]):
        p5 (Union[None, Unset, float]):
        p75 (Union[None, Unset, float]):
        p95 (Union[None, Unset, float]):
        values (Union[Unset, list[float]]):
    """

    name: str
    max_: Union[None, Unset, float] = UNSET
    mean: Union[None, Unset, float] = UNSET
    median: Union[None, Unset, float] = UNSET
    min_: Union[None, Unset, float] = UNSET
    p25: Union[None, Unset, float] = UNSET
    p5: Union[None, Unset, float] = UNSET
    p75: Union[None, Unset, float] = UNSET
    p95: Union[None, Unset, float] = UNSET
    values: Union[Unset, list[float]] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        name = self.name

        max_: Union[None, Unset, float]
        if isinstance(self.max_, Unset):
            max_ = UNSET
        else:
            max_ = self.max_

        mean: Union[None, Unset, float]
        if isinstance(self.mean, Unset):
            mean = UNSET
        else:
            mean = self.mean

        median: Union[None, Unset, float]
        if isinstance(self.median, Unset):
            median = UNSET
        else:
            median = self.median

        min_: Union[None, Unset, float]
        if isinstance(self.min_, Unset):
            min_ = UNSET
        else:
            min_ = self.min_

        p25: Union[None, Unset, float]
        if isinstance(self.p25, Unset):
            p25 = UNSET
        else:
            p25 = self.p25

        p5: Union[None, Unset, float]
        if isinstance(self.p5, Unset):
            p5 = UNSET
        else:
            p5 = self.p5

        p75: Union[None, Unset, float]
        if isinstance(self.p75, Unset):
            p75 = UNSET
        else:
            p75 = self.p75

        p95: Union[None, Unset, float]
        if isinstance(self.p95, Unset):
            p95 = UNSET
        else:
            p95 = self.p95

        values: Union[Unset, list[float]] = UNSET
        if not isinstance(self.values, Unset):
            values = self.values

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({"name": name})
        if max_ is not UNSET:
            field_dict["max"] = max_
        if mean is not UNSET:
            field_dict["mean"] = mean
        if median is not UNSET:
            field_dict["median"] = median
        if min_ is not UNSET:
            field_dict["min"] = min_
        if p25 is not UNSET:
            field_dict["p25"] = p25
        if p5 is not UNSET:
            field_dict["p5"] = p5
        if p75 is not UNSET:
            field_dict["p75"] = p75
        if p95 is not UNSET:
            field_dict["p95"] = p95
        if values is not UNSET:
            field_dict["values"] = values

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        name = d.pop("name")

        def _parse_max_(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        max_ = _parse_max_(d.pop("max", UNSET))

        def _parse_mean(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        mean = _parse_mean(d.pop("mean", UNSET))

        def _parse_median(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        median = _parse_median(d.pop("median", UNSET))

        def _parse_min_(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        min_ = _parse_min_(d.pop("min", UNSET))

        def _parse_p25(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        p25 = _parse_p25(d.pop("p25", UNSET))

        def _parse_p5(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        p5 = _parse_p5(d.pop("p5", UNSET))

        def _parse_p75(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        p75 = _parse_p75(d.pop("p75", UNSET))

        def _parse_p95(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        p95 = _parse_p95(d.pop("p95", UNSET))

        values = cast(list[float], d.pop("values", UNSET))

        system_metric_info = cls(
            name=name, max_=max_, mean=mean, median=median, min_=min_, p25=p25, p5=p5, p75=p75, p95=p95, values=values
        )

        system_metric_info.additional_properties = d
        return system_metric_info

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Sequence
from os import getenv
from typing import Optional, Union

from pydantic import UUID4

from galileo.base import BaseClientModel
from galileo.config import GalileoPythonConfig
from galileo.projects import Projects
from galileo.resources.api.protect import (
    create_stage_projects_project_id_stages_post,
    get_stage_projects_project_id_stages_get,
    pause_stage_projects_project_id_stages_stage_id_put,
    update_stage_projects_project_id_stages_stage_id_post,
)
from galileo.resources.models.rulesets_mixin import RulesetsMixin as APIRulesetsMixin
from galileo.resources.models.stage_db import StageDB as APIStageDB
from galileo.resources.models.stage_with_rulesets import StageWithRulesets as APIStageWithRulesets
from galileo.resources.types import UNSET
from galileo.utils.catch_log import DecorateAllMethods
from galileo_core.schemas.protect.ruleset import Ruleset, RulesetsMixin
from galileo_core.schemas.protect.stage import StageDB, StageType, StageWithRulesets
from galileo_core.utils.name import ts_name


def _get_project_id(
    project_id: Optional[Union[str, UUID4]] = None,
    project_name: Optional[str] = None,
    config: Optional[GalileoPythonConfig] = None,
) -> str:
    """
    Resolves project ID from either project_id or project_name.
    """
    if project_id:
        return str(project_id)

    # Load the project name from environment variable if not provided
    project_name = project_name if project_name else getenv("GALILEO_PROJECT")

    if project_name:
        project = Projects(config=config).get(name=project_name)
        if not project:
            raise ValueError(f"Project with name '{project_name}' not found.")
        return str(project.id)

    raise ValueError("Either project_id or project_name must be provided.")


def _get_stage_id(
    stage_id: Optional[Union[str, UUID4]] = None,
    stage_name: Optional[str] = None,
    project_id: Optional[Union[str, UUID4]] = None,
    config: Optional[GalileoPythonConfig] = None,
) -> str:
    """
    Resolves stage ID from either stage_id or stage_name.
    If stage_name is provided, it will look up the stage within the specified project.
    """
    if stage_id:
        return str(stage_id)
    elif stage_name:
        stage = Stages(config=config).get(project_id=project_id, stage_name=stage_name)
        if not stage:
            raise ValueError(f"Stage with name '{stage_name}' not found.")
        return str(stage.id)
    else:
        raise ValueError("Either stage_id or stage_name must be provided.")


class Stages(BaseClientModel, DecorateAllMethods):
    def create(
        self,
        project_id: Optional[Union[str, UUID4]] = None,
        project_name: Optional[str] = None,
        name: Optional[str] = None,
        stage_type: StageType = StageType.local,
        pause: bool = False,
        prioritized_rulesets: Optional[Sequence[Ruleset]] = None,
        description: Optional[str] = None,
    ) -> StageDB:
        actual_project_id: str = _get_project_id(project_id=project_id, project_name=project_name, config=self.config)

        actual_name = name or ts_name("stage")

        request = StageWithRulesets(
            name=actual_name,
            project_id=str(actual_project_id),
            type=stage_type,
            paused=pause,
            description=description,
            prioritized_rulesets=prioritized_rulesets or [],
        )

        request_dict = request.model_dump(mode="json")
        request_dict["prioritized_rulesets"] = request_dict.pop("rulesets", [])
        body = APIStageWithRulesets.from_dict(request_dict)

        response = create_stage_projects_project_id_stages_post.sync(
            project_id=str(actual_project_id), client=self.client, body=body
        )

        if isinstance(response, APIStageDB):
            return StageDB.model_validate(response.to_dict())
        return response

    def get(
        self,
        project_id: Optional[Union[str, UUID4]] = None,
        project_name: Optional[str] = None,
        stage_id: Optional[Union[str, UUID4]] = None,
        stage_name: Optional[str] = None,
    ) -> StageDB:
        actual_project_id: str = _get_project_id(project_id=project_id, project_name=project_name, config=self.config)

        if not stage_id and not stage_name:
            raise ValueError("Either stage_id or stage_name must be provided.")

        response = get_stage_projects_project_id_stages_get.sync(
            project_id=actual_project_id,
            stage_id=str(stage_id) if stage_id else UNSET,
            stage_name=stage_name if stage_name else UNSET,
            client=self.client,
        )
        if isinstance(response, APIStageDB):
            return StageDB.model_validate(response.to_dict())
        return response

    def update(
        self,
        project_id: Optional[Union[str, UUID4]] = None,
        project_name: Optional[str] = None,
        stage_id: Optional[Union[str, UUID4]] = None,
        stage_name: Optional[str] = None,
        prioritized_rulesets: Optional[Sequence[Ruleset]] = None,
    ) -> StageDB:
        actual_project_id: str = _get_project_id(project_id=project_id, project_name=project_name, config=self.config)

        actual_stage_id: str = _get_stage_id(
            stage_id=stage_id, stage_name=stage_name, project_id=actual_project_id, config=self.config
        )

        request = RulesetsMixin(prioritized_rulesets=prioritized_rulesets or [])
        request_dict = request.model_dump()
        request_dict["prioritized_rulesets"] = request_dict.pop("rulesets", [])
        body = APIRulesetsMixin.from_dict(request_dict)

        response = update_stage_projects_project_id_stages_stage_id_post.sync(
            project_id=actual_project_id, stage_id=actual_stage_id, client=self.client, body=body
        )
        if isinstance(response, APIStageDB):
            return StageDB.model_validate(response.to_dict())
        return response

    def _set_pause_state(
        self,
        pause_flag: bool,
        project_id: Optional[Union[str, UUID4]] = None,
        project_name: Optional[str] = None,
        stage_id: Optional[Union[str, UUID4]] = None,
        stage_name: Optional[str] = None,
    ) -> StageDB:
        """Sets the pause state of a stage."""
        actual_project_id: str = _get_project_id(project_id=project_id, project_name=project_name, config=self.config)

        actual_stage_id: str = _get_stage_id(
            stage_id=stage_id, stage_name=stage_name, project_id=actual_project_id, config=self.config
        )

        response = pause_stage_projects_project_id_stages_stage_id_put.sync(
            project_id=actual_project_id, stage_id=actual_stage_id, client=self.client, pause=pause_flag
        )
        if isinstance(response, APIStageDB):
            return StageDB.model_validate(response.to_dict())
        return response

    def pause(
        self,
        project_id: Optional[Union[str, UUID4]] = None,
        project_name: Optional[str] = None,
        stage_id: Optional[Union[str, UUID4]] = None,
        stage_name: Optional[str] = None,
    ) -> StageDB:
        return self._set_pause_state(
            project_id=project_id, project_name=project_name, stage_id=stage_id, stage_name=stage_name, pause_flag=True
        )

    def resume(
        self,
        project_id: Optional[Union[str, UUID4]] = None,
        project_name: Optional[str] = None,
        stage_id: Optional[Union[str, UUID4]] = None,
        stage_name: Optional[str] = None,
    ) -> StageDB:
        return self._set_pause_state(
            project_id=project_id, project_name=project_name, stage_id=stage_id, stage_name=stage_name, pause_flag=False
        )


def create_stage(
    project_id: Optional[Union[str, UUID4]] = None,
    project_name: Optional[str] = None,
    name: Optional[str] = None,
    stage_type: StageType = StageType.local,
    pause: bool = False,
    prioritized_rulesets: Optional[Sequence[Ruleset]] = None,
    description: Optional[str] = None,
) -> Optional[StageDB]:
    """Creates a new stage.

    Args:
        project_id: The ID of the project.
        project_name: Name of the project. If `project_id` is not provided,
                      this will be used to look up the project.
        name: Name of the stage. Defaults to a generated name.
        stage_type: Type of the stage.
        pause: Whether the stage should be created in a paused state.
        prioritized_rulesets: List of rulesets for the stage.
        description: Description for the stage.

    Returns:
        The newly created Stage.
    """
    return Stages().create(
        project_id=project_id,
        project_name=project_name,
        name=name,
        stage_type=stage_type,
        pause=pause,
        prioritized_rulesets=prioritized_rulesets,
        description=description,
    )


def get_stage(
    project_id: Optional[Union[str, UUID4]] = None,
    project_name: Optional[str] = None,
    stage_id: Optional[Union[str, UUID4]] = None,
    stage_name: Optional[str] = None,
) -> Optional[StageDB]:
    """Retrieves a stage by its ID or name, within a given project.

    Args:
        project_id: ID of the project.
        project_name: Name of the project. If `project_id` is not provided,
                      this will be used to look up the project.
        stage_id: ID of the stage to retrieve.
        stage_name: Name of the stage to retrieve. If `stage_id` is not provided,
                    this will be used (in conjunction with project ID/name).

    Returns:
        The fetched Stage.
    """
    return Stages().get(project_id=project_id, project_name=project_name, stage_id=stage_id, stage_name=stage_name)


def update_stage(
    project_id: Optional[Union[str, UUID4]] = None,
    project_name: Optional[str] = None,
    stage_id: Optional[Union[str, UUID4]] = None,
    stage_name: Optional[str] = None,
    prioritized_rulesets: Optional[Sequence[Ruleset]] = None,
) -> Optional[StageDB]:
    """Updates a stage's rulesets, creating a new version.

    Args:
        project_id: ID of the project.
        project_name: Name of the project. If `project_id` is not provided,
                      this will be used to look up the project.
        stage_id: ID of the stage to update.
        stage_name: Name of the stage to update. If `stage_id` is not provided,
                    this will be used (in conjunction with project ID/name).
        prioritized_rulesets: New list of prioritized rulesets for the stage.
                              If `None`, effectively clears existing rulesets.

    Returns:
        The updated Stage.
    """
    return Stages().update(
        project_id=project_id,
        project_name=project_name,
        stage_id=stage_id,
        stage_name=stage_name,
        prioritized_rulesets=prioritized_rulesets,
    )


def pause_stage(
    project_id: Optional[Union[str, UUID4]] = None,
    project_name: Optional[str] = None,
    stage_id: Optional[Union[str, UUID4]] = None,
    stage_name: Optional[str] = None,
) -> Optional[StageDB]:
    """Pauses the specified stage.

    Pauses a stage using either its ID or name within the context of a project.

    Args:
        project_id: ID of the project containing the stage.
        project_name: Name of the project containing the stage.
                      (Used if `project_id` is not provided).
        stage_id: ID of the stage to pause.
        stage_name: Name of the stage to pause.
                    (Used if `stage_id` is not provided).

    Returns:
        The Stage with its updated pause state.
    """
    return Stages().pause(project_id=project_id, project_name=project_name, stage_id=stage_id, stage_name=stage_name)


def resume_stage(
    project_id: Optional[Union[str, UUID4]] = None,
    project_name: Optional[str] = None,
    stage_id: Optional[Union[str, UUID4]] = None,
    stage_name: Optional[str] = None,
) -> Optional[StageDB]:
    """Resumes a previously paused stage.

    Resumes a stage using either its ID or name within the context of a project.

    Args:
        project_id: ID of the project containing the stage.
        project_name: Name of the project containing the stage.
                      (Used if `project_id` is not provided).
        stage_id: ID of the stage to resume.
        stage_name: Name of the stage to resume.
                    (Used if `stage_id` is not provided).

    Returns:
        The Stage with its updated pause state.
    """
    return Stages().resume(project_id=project_id, project_name=project_name, stage_id=stage_id, stage_name=stage_name)



import datetime
from collections.abc import Mapping
from typing import Any, Literal, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.log_records_date_filter_operator import LogRecordsDateFilterOperator
from ..types import UNSET, Unset

T = TypeVar("T", bound="LogRecordsDateFilter")


@_attrs_define
class LogRecordsDateFilter:
    """
    Attributes:
        column_id (str): ID of the column to filter.
        operator (LogRecordsDateFilterOperator):
        value (datetime.datetime):
        type_ (Union[Literal['date'], Unset]):  Default: 'date'.
    """

    column_id: str
    operator: LogRecordsDateFilterOperator
    value: datetime.datetime
    type_: Union[Literal["date"], Unset] = "date"
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        column_id = self.column_id

        operator = self.operator.value

        value = self.value.isoformat()

        type_ = self.type_

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({"column_id": column_id, "operator": operator, "value": value})
        if type_ is not UNSET:
            field_dict["type"] = type_

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        column_id = d.pop("column_id")

        operator = LogRecordsDateFilterOperator(d.pop("operator"))

        value = isoparse(d.pop("value"))

        type_ = cast(Union[Literal["date"], Unset], d.pop("type", UNSET))
        if type_ != "date" and not isinstance(type_, Unset):
            raise ValueError(f"type must match const 'date', got '{type_}'")

        log_records_date_filter = cls(column_id=column_id, operator=operator, value=value, type_=type_)

        log_records_date_filter.additional_properties = d
        return log_records_date_filter

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, Optional, Union

import httpx

from galileo_core.constants.request_method import RequestMethod
from galileo_core.helpers.api_client import ApiClient

from ... import errors
from ...models.agent_span_record_with_children import AgentSpanRecordWithChildren
from ...models.http_validation_error import HTTPValidationError
from ...models.llm_span_record import LlmSpanRecord
from ...models.retriever_span_record_with_children import RetrieverSpanRecordWithChildren
from ...models.tool_span_record_with_children import ToolSpanRecordWithChildren
from ...models.workflow_span_record_with_children import WorkflowSpanRecordWithChildren
from ...types import Response


def _get_kwargs(project_id: str, span_id: str) -> dict[str, Any]:
    _kwargs: dict[str, Any] = {
        "method": RequestMethod.GET,
        "return_raw_response": True,
        "path": f"/projects/{project_id}/spans/{span_id}",
    }

    return _kwargs


def _parse_response(
    *, client: ApiClient, response: httpx.Response
) -> Optional[
    Union[
        HTTPValidationError,
        Union[
            "AgentSpanRecordWithChildren",
            "LlmSpanRecord",
            "RetrieverSpanRecordWithChildren",
            "ToolSpanRecordWithChildren",
            "WorkflowSpanRecordWithChildren",
        ],
    ]
]:
    if response.status_code == 200:

        def _parse_response_200(
            data: object,
        ) -> Union[
            "AgentSpanRecordWithChildren",
            "LlmSpanRecord",
            "RetrieverSpanRecordWithChildren",
            "ToolSpanRecordWithChildren",
            "WorkflowSpanRecordWithChildren",
        ]:
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                response_200_type_0 = AgentSpanRecordWithChildren.from_dict(data)

                return response_200_type_0
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                response_200_type_1 = WorkflowSpanRecordWithChildren.from_dict(data)

                return response_200_type_1
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                response_200_type_2 = LlmSpanRecord.from_dict(data)

                return response_200_type_2
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                response_200_type_3 = ToolSpanRecordWithChildren.from_dict(data)

                return response_200_type_3
            except:  # noqa: E722
                pass
            if not isinstance(data, dict):
                raise TypeError()
            response_200_type_4 = RetrieverSpanRecordWithChildren.from_dict(data)

            return response_200_type_4

        response_200 = _parse_response_200(response.json())

        return response_200
    if response.status_code == 422:
        response_422 = HTTPValidationError.from_dict(response.json())

        return response_422
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: ApiClient, response: httpx.Response
) -> Response[
    Union[
        HTTPValidationError,
        Union[
            "AgentSpanRecordWithChildren",
            "LlmSpanRecord",
            "RetrieverSpanRecordWithChildren",
            "ToolSpanRecordWithChildren",
            "WorkflowSpanRecordWithChildren",
        ],
    ]
]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    project_id: str, span_id: str, *, client: ApiClient
) -> Response[
    Union[
        HTTPValidationError,
        Union[
            "AgentSpanRecordWithChildren",
            "LlmSpanRecord",
            "RetrieverSpanRecordWithChildren",
            "ToolSpanRecordWithChildren",
            "WorkflowSpanRecordWithChildren",
        ],
    ]
]:
    """Get Span

    Args:
        project_id (str):
        span_id (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[HTTPValidationError, Union['AgentSpanRecordWithChildren', 'LlmSpanRecord', 'RetrieverSpanRecordWithChildren', 'ToolSpanRecordWithChildren', 'WorkflowSpanRecordWithChildren']]]
    """

    kwargs = _get_kwargs(project_id=project_id, span_id=span_id)

    response = client.request(**kwargs)

    return _build_response(client=client, response=response)


def sync(
    project_id: str, span_id: str, *, client: ApiClient
) -> Optional[
    Union[
        HTTPValidationError,
        Union[
            "AgentSpanRecordWithChildren",
            "LlmSpanRecord",
            "RetrieverSpanRecordWithChildren",
            "ToolSpanRecordWithChildren",
            "WorkflowSpanRecordWithChildren",
        ],
    ]
]:
    """Get Span

    Args:
        project_id (str):
        span_id (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[HTTPValidationError, Union['AgentSpanRecordWithChildren', 'LlmSpanRecord', 'RetrieverSpanRecordWithChildren', 'ToolSpanRecordWithChildren', 'WorkflowSpanRecordWithChildren']]
    """

    return sync_detailed(project_id=project_id, span_id=span_id, client=client).parsed


async def asyncio_detailed(
    project_id: str, span_id: str, *, client: ApiClient
) -> Response[
    Union[
        HTTPValidationError,
        Union[
            "AgentSpanRecordWithChildren",
            "LlmSpanRecord",
            "RetrieverSpanRecordWithChildren",
            "ToolSpanRecordWithChildren",
            "WorkflowSpanRecordWithChildren",
        ],
    ]
]:
    """Get Span

    Args:
        project_id (str):
        span_id (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[HTTPValidationError, Union['AgentSpanRecordWithChildren', 'LlmSpanRecord', 'RetrieverSpanRecordWithChildren', 'ToolSpanRecordWithChildren', 'WorkflowSpanRecordWithChildren']]]
    """

    kwargs = _get_kwargs(project_id=project_id, span_id=span_id)

    response = await client.arequest(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    project_id: str, span_id: str, *, client: ApiClient
) -> Optional[
    Union[
        HTTPValidationError,
        Union[
            "AgentSpanRecordWithChildren",
            "LlmSpanRecord",
            "RetrieverSpanRecordWithChildren",
            "ToolSpanRecordWithChildren",
            "WorkflowSpanRecordWithChildren",
        ],
    ]
]:
    """Get Span

    Args:
        project_id (str):
        span_id (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[HTTPValidationError, Union['AgentSpanRecordWithChildren', 'LlmSpanRecord', 'RetrieverSpanRecordWithChildren', 'ToolSpanRecordWithChildren', 'WorkflowSpanRecordWithChildren']]
    """

    return (await asyncio_detailed(project_id=project_id, span_id=span_id, client=client)).parsed



from typing import Optional

from galileo.config import GalileoPythonConfig
from galileo_core.helpers.api_client import ApiClient


class BaseClientModel:
    """
    A base class for all client data models.
    Used by the model classes to lazy load the API client.

    Args:
        config (Optional[GalileoPythonConfig], optional): The config to use. Defaults to None.
    """

    config: GalileoPythonConfig

    def __init__(self, config: Optional[GalileoPythonConfig] = None) -> None:
        if config is not None:
            self.config = config
        else:
            self.config = GalileoPythonConfig.get()

    @property
    def client(self) -> ApiClient:
        """
        Returns a fresh client from the config, ensuring the token is up-to-date.
        """
        return self.config.api_client



import logging

from galileo.base import BaseClientModel
from galileo.resources.api.data import create_llm_scorer_version_scorers_scorer_id_version_llm_post, create_scorers_post
from galileo.resources.models import ScorerTypes
from galileo.resources.models.base_scorer_version_response import BaseScorerVersionResponse
from galileo.resources.models.create_llm_scorer_version_request import CreateLLMScorerVersionRequest
from galileo.resources.models.create_scorer_request import CreateScorerRequest
from galileo.resources.models.output_type_enum import OutputTypeEnum
from galileo.resources.models.scorer_defaults import ScorerDefaults
from galileo_core.schemas.logging.step import StepType

_logger = logging.getLogger(__name__)


class Metrics(BaseClientModel):
    def create_custom_llm_metric(
        self,
        name: str,
        user_prompt: str,
        node_level: StepType = StepType.llm,
        cot_enabled: bool = True,
        model_name: str = "gpt-4.1-mini",
        num_judges: int = 3,
        description: str = "",
        tags: list[str] = [],
        output_type: OutputTypeEnum = OutputTypeEnum.BOOLEAN,
    ) -> BaseScorerVersionResponse:
        """
        Create a custom LLM metric.

        Args:
            name (str): Name of the metric.
            user_prompt (str): User prompt for the metric.
            node_level (StepType): Node level for the metric.
            cot_enabled (bool): Whether chain-of-thought is enabled.
            model_name (str): Model name to use.
            num_judges (int): Number of judges for the metric.
            description (str): Description of the metric.
            tags (List[str]): Tags associated with the metric.
            output_type (OutputTypeEnum): Output type for the metric.

        Returns:
            BaseScorerVersionResponse: Response containing the created metric details.
        """

        create_scorer_request = CreateScorerRequest(
            name=name,
            scorer_type=ScorerTypes.LLM,
            description=description,
            tags=tags,
            defaults=ScorerDefaults(model_name=model_name, num_judges=num_judges),
        )

        scorer = create_scorers_post.sync(body=create_scorer_request, client=self.client)

        scoreable_node_types = [node_level]
        version_req = CreateLLMScorerVersionRequest(
            user_prompt=user_prompt,
            scoreable_node_types=scoreable_node_types,
            cot_enabled=cot_enabled,
            output_type=output_type,
            model_name=model_name,
            num_judges=num_judges,
        )
        version_resp = create_llm_scorer_version_scorers_scorer_id_version_llm_post.sync(
            scorer_id=scorer.id, body=version_req, client=self.client
        )

        _logger.info("Created custom LLM metric: %s", name)

        return version_resp


# Public functions
def create_custom_llm_metric(
    name: str,
    user_prompt: str,
    node_level: StepType = StepType.llm,
    cot_enabled: bool = True,
    model_name: str = "gpt-4.1-mini",
    num_judges: int = 3,
    description: str = "",
    tags: list[str] = [],
    output_type: OutputTypeEnum = OutputTypeEnum.BOOLEAN,
) -> BaseScorerVersionResponse:
    """
    Create a custom LLM metric.

    Args:
        name (str): Name of the metric.
        user_prompt (str): User prompt for the metric.
        node_level (StepType): Node level for the metric.
        cot_enabled (bool): Whether chain-of-thought is enabled.
        model_name (str): Model name to use.
        num_judges (int): Number of judges for the metric.
        description (str): Description of the metric.
        tags (List[str]): Tags associated with the metric.
        output_type (OutputTypeEnum): Output type for the metric.

    Returns:
        BaseScorerVersionResponse: Response containing the created metric details.
    """
    return Metrics().create_custom_llm_metric(
        name, user_prompt, node_level, cot_enabled, model_name, num_judges, description, tags, output_type
    )
